/*
 ============================================================================
 Name        : usart.c
 Author      : m.yanagisawa
 Version     : 0.0
 Copyright   :
 Description : usart driver for NGW
               M荞݃o[W
 ============================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <avr32/io.h>
#include <avr32/ap7000.h>
#include <intc.h>

#define RXBUFSIZE 0x800
#define TXBUFSIZE 0x800

static unsigned char rxbuf[RXBUFSIZE];
static unsigned char txbuf[TXBUFSIZE];
static int rxrp,rxwp;
static int txrp,txwp;

volatile struct avr32_usart_t *usart1 = &AVR32_USART1;

// RS232CMnh
__attribute__((__interrupt__))
static void usart_int_handler(void)
{
  int dat,ofs,tail;

  // M\ȏԂ܂ŎM
  while (usart1->csr & AVR32_USART_CSR_RXRDY_MASK) {
    ofs=(rxwp+1) & (RXBUFSIZE-1);
    dat=usart1->rhr;
    if (ofs!=rxrp) {	//Mobt@ MAXłȂƂ͏
      rxbuf[rxwp]=dat;
      rxwp=ofs;
    }
  }
  tail=(txwp-txrp) & (TXBUFSIZE-1); // M\f[^擾
  if (tail) {
    // M\ȏԂ܂őM
    while (usart1->csr & AVR32_USART_CSR_TXRDY_MASK) {
      usart1->thr=txbuf[txrp];
      txrp=(txrp+1) & (TXBUFSIZE-1);
      tail--;
      if (!tail) break;
    }
  }
  if (!tail) usart1->idr=AVR32_USART_IDR_TXRDY_MASK;
}

// 荞݂gȂM
void send_direct(int c)
{
    while ((usart1->csr & AVR32_USART_CSR_TXRDY_MASK)==0) ;
    usart1->thr=c;
}

// 1byteM
int recv_byte(void)
{
	int tail,res;

	tail=(rxwp-rxrp) & (RXBUFSIZE-1);
	if (!tail) return -1;
	res=rxbuf[rxrp];
	rxrp=(rxrp+1) & (RXBUFSIZE-1);
	return res;
}

// 1byteM
// Mobt@֏ŁAM\荞݂
int send_byte(int c)
{
	int tail;

	tail=(txwp-txrp) & (TXBUFSIZE-1);
	if (tail>=TXBUFSIZE-1) return -1;
	txbuf[txwp]=c;
	txwp=(txwp+1) & (TXBUFSIZE-1);
	usart1->ier = AVR32_USART_IER_TXRDY_MASK;
	return 0;
}

// RS232C[hUSART1
// 115200bps N81ŐݒB
static void open_usart(void)
{
	// pio
  	volatile struct avr32_pio_t *pioa=&AVR32_PIOA;
  	unsigned int mask;

  	mask=(1<<AVR32_USART1_RXD_0_PIN) | (1<<AVR32_USART1_TXD_0_PIN);
    pioa->pdr=mask;	// PA17,18 (TXD1,RXD1)
    pioa->pudr=mask;  // pull up disable
    pioa->asr=mask;   // PA select

	usart1->mr  = (AVR32_USART_CHRL_8 << AVR32_USART_CHRL_OFFSET) |
				 (AVR32_USART_MODE_NORMAL << AVR32_USART_MODE_OFFSET)|
				 (AVR32_USART_MR_USCLKS_MCK_DIV << AVR32_USART_MR_USCLKS_OFFSET) |
				 (AVR32_USART_PAR_NONE << AVR32_USART_MR_PAR_OFFSET) |
				 (AVR32_USART_NBSTOP_1 << AVR32_USART_NBSTOP_OFFSET)|
				 (AVR32_USART_CHMODE_NORMAL << AVR32_USART_CHMODE_OFFSET)|
				 (AVR32_USART_OVER_X16 << AVR32_USART_OVER_OFFSET);
	rxwp=rxrp=txrp=txwp=0;
// PBA=30MHz MCK=PBA/8 OVER=0(x16) 115200bps
// CD=2: 30^6/8/16/CD=117187.5 (117187.5/115200-1*100=1.72%)
    usart1->brgr=2;
	usart1->cr =  AVR32_USART_CR_RXEN_MASK | AVR32_USART_CR_TXEN_MASK;
	usart1->ier = AVR32_USART_IER_RXRDY_MASK;
}

// USART1Zbg
static void usart_reset(void)
{
	// SĂ̊荞݂֎~
	usart1->idr = 0xFFFFFFFF;

	// [hWX^ZbgB
	usart1->mr = 0;
	usart1->rtor = 0;
	usart1->ttgr = 0;

    // ׂĂZbgB
	usart1->cr = AVR32_USART_CR_RSTRX_MASK   |
	            AVR32_USART_CR_RSTTX_MASK   |
	            AVR32_USART_CR_RSTSTA_MASK  |
	            AVR32_USART_CR_RSTIT_MASK   |
	            AVR32_USART_CR_RSTNACK_MASK |
	            AVR32_USART_CR_DTRDIS_MASK  |
	            AVR32_USART_CR_RTSDIS_MASK;
}

void init_usart(void)
{
  usart_reset();
  /* Setup the interrupt handler, handled by newlib */
//  INTC_set_interrupts_base( (void *) AVR32_INTC_ADDRESS );
  /* INTC_register_intterupt( (__int_handler) {func}, {GROUP}, {LINE}); */
  INTC_register_interrupt( &usart_int_handler, AVR32_USART1_IRQ, AVR32_INTC_INT0);
  open_usart();
}
