/*
 ============================================================================
 Name        : test.c
 Author      :
 Version     : 1.0
 Copyright   :
 Description : AVR32 NGW TEST PROGRAM
 ============================================================================
 */

#include <stdio.h>
#include <stdlib.h>
#include <avr32/io.h>
#include <avr32/ap7000.h>
#include <intc.h>

#define LED_SYS 0x01
#define LED_A   0x02
#define LED_B   0x04

// led port
// PA16 : SYS
// PA18 : A
// PE19 : B
void init_ports(void)
{
  volatile struct avr32_pio_t *pioa = &AVR32_PIOA;
  volatile struct avr32_pio_t *pioe = &AVR32_PIOE;

  pioa->per=0x00090000;
  pioe->per=0x00080000;
  pioa->oer=0x00090000;
  pioe->oer=0x00080000;
  pioa->sodr=0x00090000; // off
  pioe->sodr=0x00080000; // off

}

void disp_led(int led,int onoff)
{
  volatile struct avr32_pio_t *pioa = &AVR32_PIOA;
  volatile struct avr32_pio_t *pioe = &AVR32_PIOE;
  if (led & LED_SYS) {
    if (onoff) pioa->codr=0x00010000;
    else	   pioa->sodr=0x00010000;
  }
  if (led & LED_A) {
    if (onoff) pioa->codr=0x00080000;
    else	   pioa->sodr=0x00080000;
  }
  if (led & LED_B) {
    if (onoff) pioe->codr=0x00080000;
    else	   pioe->sodr=0x00080000;
  }
}


// 20MBmۂwrite&verify
// SDRAM = 32MBȂ̂ŔȏcheckokB
int memory_test(void)
{
	unsigned long *p;
	int i;
#define CHECKSIZE (20*1024*1024)

	p=(unsigned long*)malloc(CHECKSIZE);
	for (i=0;i<(CHECKSIZE >> 2);i++) {
	  p[i]=i;
	}
	for (i=0;i<(CHECKSIZE >> 2);i++) {
	  if (p[i]!=i) break;
	}
	free(p);
	if (i!=(CHECKSIZE >> 2)) return -1;
	else					 return 0;
}

int recv_byte(void);
int send_byte(int c);
void init_usart(void);
void send_direct(int c);

char testdata[]="ABCDEF\r\n";  //@l̕ϐ

static void send_line(char *s)
{
   while (*s) {
	   while (send_byte(*s)<0) ;
	   s++;
   }
}

int main(void) {
	int res;

	init_ports();
	INTC_init_interrupts();
	init_usart();
	Enable_global_interrupt();
	testdata[4]='0'; // lϐ̏
	for (res=0;res<sizeof(testdata);res++)  send_direct(testdata[res]);
	while (1) {
	   res=recv_byte();
	   if (res>=0) {
		 while (send_byte(res)<0) ;
	   }
	   if (res=='#') break;
	}
	send_line("\r\nTESTING SDRAM MEMORY...\r\n");
	res=memory_test();
	if (res) {
	    send_line("MEMORY TEST ERROR!!\r\n");
		disp_led(LED_SYS,1);
	} else {
		send_line("MEMORY TEST OK!!\r\n");
		disp_led(LED_A,1);
	}

	while (1) ; // term.
}
